/**
 * @fileoverview Conditionally rendered text nodes should be wrapped in an element (generally a <span>), otherwise Google Translate will cause a browser error.
 * @author alistair-coup
 */
'use strict';

/** @type {import('eslint').Rule.RuleModule} */
module.exports = {
  meta: {
    type: 'problem',
    docs: {
      description:
        'Conditionally rendered text nodes should be wrapped in an element (generally a <span>), otherwise Google Translate can cause a browser error.',
    },
    schema: [],
    messages: {
      unexpected:
        'When rendered as a direct child of a JSX element, conditionally rendered text nodes should be wrapped in an element (generally a `<span>`) to prevent Google Translate causing a browser error while manipulating the DOM.',
    },
  },

  create(context) {
    function isConditionallyRendered(node) {
      if (!node.parent) {
        return false;
      }
      return (
        node.parent.type === 'ConditionalExpression' ||
        node.parent.type === 'LogicalExpression'
      );
    }

    function isChildOfJSXElementWithSiblings(node) {
      return (
        node.parent.parent &&
        node.parent.parent.type === 'JSXExpressionContainer' &&
        node.parent.parent.parent &&
        node.parent.parent.parent.type === 'JSXElement' &&
        node.parent.parent.parent.children.length > 1
      );
    }

    function functionWasCalled(node) {
      if (!node.parent || node.parent.type === 'ReturnExpression') {
        return false;
      } else if (node.parent.type === 'CallExpression') {
        return true;
      } else {
        return functionWasCalled(node.parent);
      }
    }

    return {
      // text nodes (string literals, or other literals that are coerced to strings)
      Literal(node) {
        if (
          isConditionallyRendered(node) &&
          isChildOfJSXElementWithSiblings(node)
        ) {
          context.report({
            node,
            messageId: 'unexpected',
          });
        }
      },
      // text nodes generated by calling toLocaleString
      Identifier(node) {
        if (node.name === 'toLocaleString' && functionWasCalled(node)) {
          const callExpression = node.parent.parent;
          if (
            isConditionallyRendered(callExpression) &&
            isChildOfJSXElementWithSiblings(callExpression)
          ) {
            context.report({
              node: callExpression,
              messageId: 'unexpected',
            });
          }
        }
      },
    };
  },
};
